--[[
	Logitech G-15 Buttons Addon - Eyyub Volkan ektimur (eyyubvolkan@yahoo.com)
	
	This add-on provides groups of action buttons. But optimized for Logitech G-15 keyboard users. You can create upto 24 buttons per group and
	can change starting id of buttons. You can also change the row count, offset, scale and someother options. You can create a group of buttons 
	and move it where ever you want. in actuall, there are only 3 groups of buttons (like g-15 have).

Todo List
---------------
	* button textures

Change Log
---------------
0.50b
	* alwaysShowButtons fixed
	* 3 groups of buttons
	* options form added
	* save/load support added
	* the first beta of addon
0.10b
	* code start
--]]
	
-- objG15 and other variables are defined at Definitions.lua ---------------------------------------------------------
-- objG15 and other variables are defined at Definitions.lua ---------------------------------------------------------
-- objG15 and other variables are defined at Definitions.lua ---------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------
-- load group data from saved data
function objG15.LoadFrom(group)
 local loadFrom=objG15SavedDatas.ButtonGroups[group.id];
 
 if (loadFrom~=null) then
	group.buttonCount=loadFrom.buttonCount;
	group.rowCount=loadFrom.rowCount;
	group.startingButtonID=loadFrom.startingButtonID;
	group.scale=loadFrom.scale;
	group.alwaysShowButtons=loadFrom.alwaysShowButtons;
	group.spacePerRow=loadFrom.spacePerRow;
	group.spaceAfterEveryXRow=loadFrom.spaceAfterEveryXRow;
	group.spacePerColumn=loadFrom.spacePerColumn;
	group.spaceAfterEveryXColumn=loadFrom.spaceAfterEveryXColumn;
	group.activated=loadFrom.activated;
	group.lockMovement=loadFrom.lockMovement;
	
	if (loadFrom.framePosition~=null) then
		group.framePosition.point=loadFrom.framePosition.point;
		group.framePosition.relativePoint=loadFrom.framePosition.relativePoint;
		group.framePosition.xOfs=loadFrom.framePosition.xOfs;
		group.framePosition.yOfs=loadFrom.framePosition.yOfs;
		end;	
	end;
end;
----------------------------------------------------------------------------------------------------------------------
-- save group data to save data
function objG15.SaveGroup(group)
 if (objG15SavedDatas.ButtonGroups[group.id]==null) then
	objG15SavedDatas.ButtonGroups[group.id]={};
 end;
 
 local saveTo=objG15SavedDatas.ButtonGroups[group.id];
 
 saveTo.buttonCount=group.buttonCount;
 saveTo.rowCount=group.rowCount;
 saveTo.startingButtonID=group.startingButtonID;
 saveTo.scale=group.scale;
 saveTo.alwaysShowButtons=group.alwaysShowButtons;
 saveTo.spacePerRow=group.spacePerRow;
 saveTo.spaceAfterEveryXRow=group.spaceAfterEveryXRow;
 saveTo.spacePerColumn=group.spacePerColumn;
 saveTo.spaceAfterEveryXColumn=group.spaceAfterEveryXColumn;
 saveTo.activated=group.activated;
 saveTo.lockMovement=group.lockMovement;
 
 saveTo.framePosition={};
 saveTo.framePosition.point=group.framePosition.point;
 saveTo.framePosition.relativePoint=group.framePosition.relativePoint;
 saveTo.framePosition.xOfs=group.framePosition.xOfs;
 saveTo.framePosition.yOfs=group.framePosition.yOfs;
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.onLoaded()
 objG15.Screen.debug=objG15.DEBUGMODE;
 objG15.Screen:Print("Welcome to G15 Buttons add-on");
 
 objG15.doLocalization() ;	-- function defined at definitions.lua
 
 -- Register Slash Commands
 SlashCmdList["G15BUTTONS"]=objG15.OnSlashCommand;
	SLASH_G15BUTTONS1="/g15"; 
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.VariablesLoaded()
 -- after load
 objG15SavedDatas.Version=CURRENTSAVEVERSION;
 
 for i=1,objG15.ButtonGroups.count,1 do
	objG15.LoadFrom(objG15.ButtonGroups[i])
 end;  
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.PlayerLogin()
 -- create buttons here to make work key-binding functions
 for i=1,objG15.ButtonGroups.count,1 do
	objG15.CreateButtonGroup(objG15.ButtonGroups[i]);
 end; 
end;
----------------------------------------------------------------------------------------------------------------------
-- returns group from frame
function objG15.GroupFromFrame(bar)
 local index=bar.groupId;
 
 if (index~=null) then
	return objG15.ButtonGroups[index];
 else
	return null;
 end;
end;
----------------------------------------------------------------------------------------------------------------------
-- returns group from button
function objG15.GroupFromButton(btn)
 local index=btn.groupId;
 
 if (index~=null) then
	return objG15.ButtonGroups[index];
 else
	return null;
 end;
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.onButtonShow(btn)
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.onButtonHide(btn)
 local group=objG15.GroupFromButton(btn);

 if (group~=null and group.alwaysShowButtons) then
	btn:Show();
 end;
end;
----------------------------------------------------------------------------------------------------------------------
function objG15.onButtonHint(btn)
 local group=objG15.GroupFromButton(btn);

 ActionButton_SetTooltip();
 if (group~=null) then
	local keybind=GetBindingKey("G15BUTTONS_BINDINGS_"..group.id.."_"..btn.index);
	if (keybind~=null) then
		GameTooltip:AddLine("|cffffcc00");
		GameTooltip:AddLine("|cffffcc00 "..keybind);
	end;
 end;
end;
----------------------------------------------------------------------------------------------------------------------
-- clears group bindings
function objG15.ClearBindings(group)
 for i=1,24,1 do
	local KeyBound=GetBindingKey("G15BUTTONS_BINDINGS_"..group.id.."_"..i);
	if (KeyBound~=null) then
		SetOverrideBinding(UIParent,false,KeyBound);
		end;
 end;
end;
----------------------------------------------------------------------------------------------------------------------
-- deletes/pools the created frames/buttons
function objG15.DeleteButtonGroup(group)
 objG15.ClearBindings(group);
 
 if (group.createdBar~=null) then
	objG15.pooledObjects.frames:Add(group.createdBar);
	group.createdBar:SetParent(UIParent);
	group.createdBar:SetPoint("TOPLEFT",UIParent,"TOPRIGHT",-1000,-1000);
	group.createdBar:Hide();
	group.createdBar.groupId=null;
	group.createdBar=null;
 end;
 
 for i=1,group.createdButtons:GetCount(),1 do
	local btn=group.createdButtons:GetItem(i);
	objG15.pooledObjects.buttons:Add(btn);
	btn:SetParent(UIParent);
	btn:SetAttribute("*action*",0);
	btn:SetPoint("TOPLEFT",UIParent,"TOPRIGHT",-1000,-1000);
	btn:Disable();
	btn:Hide();
	btn.groupId=null;
 end;
 group.createdButtons:Clear();
end;
----------------------------------------------------------------------------------------------------------------------
-- if an empty frame found in the pool then returns it, otherwise creates a new frame and returns it
function objG15.CreateFrame()
 if (objG15.pooledObjects.frames:GetCount()>0) then
	local result=objG15.pooledObjects.frames:GetItem(objG15.pooledObjects.frames:GetCount());
	objG15.pooledObjects.frames:Remove(objG15.pooledObjects.frames:GetCount());
	return result;
	else
	objG15.LastGiveId=objG15.LastGiveId+1;
	return CreateFrame("Frame",objG15.UIObjectPrefix..objG15.UIBarPrefix..objG15.LastGiveId,UIParent,"G15Buttons_BarTemplate");	
	end;
end;
----------------------------------------------------------------------------------------------------------------------
-- if an empty button found in the pool then returns it, otherwise creates a new button and returns it
function objG15.CreateButton(parent)
 if (objG15.pooledObjects.buttons:GetCount()>0) then
	local result=objG15.pooledObjects.buttons:GetItem(objG15.pooledObjects.buttons:GetCount());
	objG15.pooledObjects.buttons:Remove(objG15.pooledObjects.buttons:GetCount());
	return result;
	else
	objG15.LastGiveId=objG15.LastGiveId+1;
	return CreateFrame("CheckButton",objG15.UIObjectPrefix..objG15.UIButtonPrefix..objG15.LastGiveId,parent,"G15Buttons_ButtonTemplate");			
	end;
end;
----------------------------------------------------------------------------------------------------------------------
-- re-create/create all group
function objG15.CreateButtonGroup(group)
 objG15.DeleteButtonGroup(group);
 
 if (group.activated and group.buttonCount>0 and group.rowCount>0) then
	local buttonPerRow=math.round(group.buttonCount/group.rowCount);
	local moveBar=null;
	local createdButtonCount=0;
	local buttonID=group.startingButtonID;
	local Anchor="TOPLEFT";
	local AnchorTo="TOPRIGHT";
	local AnchorToObject=null;
	local RowsFirstButton=null;
	local offsetX=0;
	local offsetY=0;
	
	moveBar=objG15.CreateFrame();
	moveBar:Show();
	group.createdBar=moveBar;
	moveBar.groupId=group.id;
	if (group.framePosition.yOfs>0) then
		group.framePosition.yOfs=0;
	end;
	if (group.framePosition.xOfs<0) then
		group.framePosition.xOfs=0;
	end;
	moveBar:SetPoint(group.framePosition.point,UIParent,group.framePosition.relativePoint,group.framePosition.xOfs,group.framePosition.yOfs);
				
	if (group.lockMovement) then
		moveBar:Hide();
		else
		moveBar:Show();
		end;
				
	if (tonumber(group.spacePerColumn)==null) then
		group.spacePerColumn=0;
		end;
	if (tonumber(group.spacePerRow)==null) then
		group.spacePerRow=0;
		end;	
	
	AnchorToObject=moveBar;
	for row=1,group.rowCount,1 do 
		for column=1,buttonPerRow,1 do
			-- if you give moveBar for parent and if the move bar is invisible at the begining (loading) , buttons do not appears on screen, don't know why ?
			local btn=objG15.CreateButton(UIParent);
			createdButtonCount=createdButtonCount+1;

			btn:Enable();
			btn:Show();
			btn:SetPoint(Anchor,AnchorToObject,AnchorTo,offsetX,offsetY);
			btn:SetAttribute("*action*",buttonID);
			btn:SetScale(group.scale);
			btn.groupId=group.id;
			btn.index=createdButtonCount;
			group.createdButtons:Add(btn);
			
			local KeyBound=GetBindingKey("G15BUTTONS_BINDINGS_"..group.id.."_"..createdButtonCount);
			if (KeyBound~=null) then
				SetOverrideBindingClick(moveBar,false,KeyBound,btn:GetName());
				end;
			
			if (column==1) then
				RowsFirstButton=btn;
			end;
			
			AnchorToObject=btn;
			Anchor="TOPLEFT";
			AnchorTo="TOPRIGHT";
			offsetX=objG15.btnOffsetX;
			offsetY=0;
		
			if (group.spaceAfterEveryXColumn>0 and (column%group.spaceAfterEveryXColumn)==0) then
				offsetX=offsetX+group.spacePerColumn;
			end;
		
			buttonID=buttonID+1;
			
			if (createdButtonCount>=group.buttonCount) then
				break;
			end;
		end; -- for
		AnchorToObject=RowsFirstButton;
		Anchor="TOPLEFT";
		AnchorTo="BOTTOMLEFT";
		offsetX=0;
		offsetY=-objG15.btnOffsetY;
		if (group.spaceAfterEveryXRow>0 and (row%group.spaceAfterEveryXRow)==0) then
			offsetY=offsetY-group.spacePerRow;
		end;
	end; -- for
	end; -- if
end;
----------------------------------------------------------------------------------------------------------------------
-- slash commands
function objG15.OnSlashCommand(msg)
 if (objG15.DEBUGMODE and msg=="1") then
	objG15.CreateButtonGroup(objG15.ButtonGroups[1]);
 elseif (objG15.DEBUGMODE and msg=="2") then
	objG15.DeleteButtonGroup(objG15.ButtonGroups[1]);
 else
	objG15.ShowOptions();	-- function defined at OptionsForm.lua
 end;
end
----------------------------------------------------------------------------------------------------------------------
-- happens when the anchor frame moved
function objG15.onFrameMoved(sender)
 local group=objG15.GroupFromFrame(sender);

 if (group~=null) then		
	group.framePosition.point,_,group.framePosition.relativePoint,group.framePosition.xOfs,group.framePosition.yOfs=group.createdBar:GetPoint();
	objG15.SaveGroup(group);
 end;
end;
----------------------------------------------------------------------------------------------------------------------